@@ -1,4 +1,5 @@
 from ..data import tmp
+from . import logger
 import re
 
 
@@ -7,6 +8,9 @@ class NewDict(dict):
     def __init__(self, *args, **kwargs):
         super(NewDict, self).__init__(*args, **kwargs)
 
+    def getdefault(self, args, default):
+        return super().get(args, default)
+
     def get(self, *args):
         if len(args) <= 1:
             return super().get(args[0])
@@ -32,6 +36,9 @@ class NewDict(dict):
                 _json1[k] = self.replace_dict(_json1[k], _json2[k])
         return _json1
 
+    def merge_dict(self, json: dict):
+        return self.replace_dict(self, json)
+
 
 def singleton(cls):
     """
