@@ -8,7 +8,12 @@ from .configuration.request import requestBase
 from rediscluster import RedisCluster
 from redis import StrictRedis
 from autoTestScheme.configuration import robot
+from selenium import webdriver
+from selenium.webdriver.chrome.options import Options
+from selenium.webdriver.chrome.service import Service
+from webdriver_manager.chrome import ChromeDriverManager
 import os, copy
+import traceback
 
 
 class Report(dict):
@@ -23,15 +28,18 @@ class Report(dict):
 
 class BaseDynaconf(dynaconf.Dynaconf):
     _hook_list = {}
+    _hook_client_list = {}
+    _hook_close_list = {}
     _client_list = {}
+    _fixture_list = []
     _report = Report()
-    default_settings = {'current_tag': 'debug', 'test_tags': ['info'], 'test_case': 'all', 'is_debug': False, 'process_num': 1,
-                        'tag_name_list': {'all': '其他'}, 'tag_env_list': {'all': 'all'}, 'is_locust': False}
+    default_settings = {'current_env': 'debug', 'env': 'debug', 'test_case': 'all', 'is_debug': False,
+                        'env_name': '测试环境', 'is_locust': False}
 
     def __init__(self, *args, **kwargs):
         super(BaseDynaconf, self).__init__(*args, **kwargs)
-        self._first_register = False
-        self._end_register = False
+        self._first_session = False
+        self._end_session = False
         self.init_run_config()
 
     def init_run_config(self):
@@ -40,20 +48,30 @@ class BaseDynaconf(dynaconf.Dynaconf):
         for key, value in self.default_settings.items():
             if key not in self.run:
                 setattr(self.run, key, value)
-        self._report.env.append(['环境:', self.get_env_name()])
-        tag_name = ';'.join([self.run.tag_name_list[i] for i in self.run.test_tags])
-        self._report.env.append(['标签:', tag_name])
-        self._report.env.append(['执行用例:', self.run.test_case])
-        self._report.env.append(['并行数:', self.run.process_num])
+        self._report.env.append(['环境:', self.run.env_name])
 
+        tag_name = []
+        self._report.env.append(['执行用例:', self.run.test_case])
         self.register_hook('redis', self.get_redis)
         self.register_hook('request', self.get_session)
         self.register_hook('sql', self.get_sql)
         self.register_hook('feishu', self.get_feishu)
         self.register_hook('ws', self.get_websocket)
+        self.register_hook('browser', self.get_browsert)
+        self.register_hook_close('browser', self.close_browsert)
 
-    def get(self, name, *args, **kwargs):
-        return self.__getattr__(name)
+    def close_browsert(self, browsert):
+        logger.info('关闭浏览器驱动')
+        browsert.close()
+        browsert.quit()
+
+    def get(self, name, default_value=None):
+        try:
+            return self.__getattr__(name)
+        except Exception as e:
+            if default_value is not None:
+                return default_value
+            raise e
 
     def __getattr__(self, item: str):
         if item == '_report':
@@ -67,10 +85,25 @@ class BaseDynaconf(dynaconf.Dynaconf):
         for name, func in self._hook_list.items():
             if item.startswith(name):
                 if item not in self._client_list:
+                    if name not in self._hook_client_list:
+                        self._hook_client_list[name] = []
                     self._client_list[item] = func(item, result)
+                    self._hook_client_list[name].append(self._client_list[item])
                 return self._client_list[item]
         return result
 
+    def register_hook_close(self, name, func):
+        self._hook_close_list[name] = func
+
+    def execute_all_close_hook(self):
+        for name, func in self._hook_close_list.items():
+            if name in self._hook_client_list:
+                for i in self._hook_client_list[name]:
+                    try:
+                        func(i)
+                    except Exception:
+                        logger.error(f"异常:{traceback.format_exc()}")
+
     def register_hook(self, name, func):
         self._hook_list[name] = func
 
@@ -122,17 +155,22 @@ class BaseDynaconf(dynaconf.Dynaconf):
             ws.read_api_folder(*i)
         return ws
 
-    def get_env_name(self):
-        current_env = self.current_env
-        tag_name_list = self.run.get('tag_name_list', None)
-        if tag_name_list is None or current_env not in tag_name_list:
-            if self.exists('name'):
-                return self.name
-            else:
-                return current_env
-        else:
-            return self.run.tag_name_list[current_env]
+    def get_browsert(self, item, config):
+        chrome_options = Options()
+        chrome_options.add_argument('--headless')
+        chrome_options.add_argument('--disable-gpu')
+        chrome_options.add_argument('--no-sandbox')
+        chrome_options.add_argument('--blink-settings=imagesEnabled=true')
+        chrome_options.add_argument('--disable-dev-shm-usage')
+        chrome_options.add_argument('--proxy-excludeSwitcher=enable-automation')
+        for k, v in config.items():
+            chrome_options.add_argument(f'--{k}={v}')
+        path = ChromeDriverManager().install()
+        service = Service(executable_path=path)
+        browser = webdriver.Chrome(service=service, options=chrome_options)
+        browser.maximize_window()
+        return browser
 
 
 settings = BaseDynaconf(envvar_prefix=False, merge_enabled=True, environments=True, load_dotenv=True,
-                        env_switcher="ENV", root_path=constant.CONFIG_FOLDER, includes=['*.toml'])
+                        env_switcher="ENV", root_path=constant.CONFIG_FOLDER, includes=['*.toml', '*.json'])
