@@ -1,5 +1,7 @@
 import os
 import sys
+import traceback
+
 import allure
 import copy
 import time
@@ -10,6 +12,7 @@ from allure_commons._allure import Attach
 from allure import attachment_type
 from urllib.parse import urlencode
 from . import api
+from .. import conf
 from ..common import config, constant, logger
 
 
@@ -41,12 +44,13 @@ class requestBase(api.Api):
         super().__init__()
         self.setup_hook_list = []
         self.teardown_hook_list = []
+        self.except_hook_list = []
         self.logger = logger
         self.hook_switch = {}
         self.session = requests.Session()
-        self.base_url = None # 域名
-        self.catch_response = False # locust的时候使用
-        self.kwargs = {} # request配置文件内的参数
+        self.base_url = None  # 域名
+        self.catch_response = False  # locust的时候使用
+        self.kwargs = {}  # request配置文件内的参数
 
     def set_session(self, client):
         self.session = client
@@ -60,6 +64,15 @@ class requestBase(api.Api):
         self.hook_switch[hook.__name__] = True
         self.setup_hook_list.append(hook)
 
+    def register_except_hook(self, hook):
+        """
+        注册前置钩子
+        :param hook:
+        :return:
+        """
+        self.hook_switch[hook.__name__] = True
+        self.except_hook_list.append(hook)
+
     def register_teardown_hook(self, hook):
         """
         注册后置钩子
@@ -70,9 +83,9 @@ class requestBase(api.Api):
         self.teardown_hook_list.append(hook)
 
     def get_url(self, path):
-        '''
+        """
             获取网址
-        '''
+        """
         assert self.base_url is not None, '未定义base_url'
 
         if self.base_url.endswith('/'):
@@ -81,58 +94,114 @@ class requestBase(api.Api):
         return url
 
     def send(self, api_id, is_json=True, title=None, **kwargs):
-        request = self.get_api(api_id)
+        request = copy.deepcopy(self.get_api(api_id))
+        print('api:', request)
         if title is not None:
             request['title'] = title
-        request.update(kwargs)
-        response = self._excute(request)
+        request['params'] = kwargs.pop('params', {})
+        if request['method'] == "GET":
+            kwargs.pop('json', {})
+            kwargs.pop('data', {})
+        else:
+            request['json'] = self.update_dic(request.get('json', {}), kwargs.pop('json', {}))
+            request['data'] = self.update_dic(request.get('data', {}), kwargs.pop('data', {}))
+        request['headers'] = self.update_dic(request.get('headers', {}), kwargs.pop('headers', {}))
+        self.update_dic(request, kwargs)
+        response = self.execute(request)
         if is_json is True:
             return response.json()
         return response
 
+    def update_dic(self, start, end):
+        '''
+        Updater of multi-level dictionary.
+
+        Last level value is unmergable.
+        '''
+        if start is None or start == {}:
+            return end
+        for k, v in end.items():
+            if k not in start:
+                start[k] = v
+            elif isinstance(v, list) is True and isinstance(start[k], list):
+                start[k] += v
+            elif isinstance(v, dict) is True and isinstance(start[k], dict):
+                start[k] = self.update_dic(start[k], end[k])
+            else:
+                start[k] = v
+        return start
+
     def close_hook(self, key=None):
         if key is None:
             for k in self.hook_switch:
-                self.hook_switch[key] = False
-        else:
+                self.hook_switch[k] = False
+        elif key in self.hook_switch:
             self.hook_switch[key] = False
+        else:
+            raise TypeError(f'不存在的hook，hook列表:{self.hook_switch}')
 
     def open_hook(self, key=None):
         if key is None:
             for k in self.hook_switch:
-                self.hook_switch[key] = True
+                self.hook_switch[k] = True
         else:
             self.hook_switch[key] = True
 
-    def execute(self, request):
+    def execute(self, request) -> requests.Response:
         """
             发送请求
         """
+        if conf.settings.get('is_hook_logger', True) is False:
+            logger.is_close = True
         for hook in self.setup_hook_list:
             if self.hook_switch[hook.__name__] is True:
                 self.excute_kwargs(hook, request)
+        if conf.settings.get('is_hook_logger', True) is False:
+            logger.is_close = False
         request['title'] = request.get('title') if request.get('title') is not None else request.get('path')
-
         _name = request['title'].format(**request)
         with allure.step(_name):
             allure.attach(json.dumps(request, cls=MyEncoder, sort_keys=True, ensure_ascii=False, indent=4, separators=(',', ':')),
                      name="请求体", attachment_type=attachment_type.JSON)
             self.logger.debug('发送请求：{}'.format(request['title']))
-            response = self._excute(request)
+            try:
+                response = self._excute(request)
+            except Exception as e:
+                for hook in self.except_hook_list:
+                    if self.hook_switch[hook.__name__] is True:
+                        response = self.excute_kwargs(hook, request, error=e)
+                if len(self.except_hook_list) == 0:
+                    raise TypeError(f'未定义except_hook异常，异常内容:{traceback.format_exc()}')
+            if conf.settings.get('is_hook_logger', True) is False:
+                logger.is_close = True
             for hook in self.teardown_hook_list:
                 if self.hook_switch[hook.__name__] is True:
-                    self.excute_kwargs(hook, request)
+                    self.excute_kwargs(hook, request, response)
+            if conf.settings.get('is_hook_logger', True) is False:
+                logger.is_close = False
             return response
 
-    def excute_kwargs(self, hook, request):
+    def excute_kwargs(self, hook, request, response={}, error=None, num=0):
         kwarg = {}
-        kwargs = {'request': request, 'session': self.session, 'kwargs': self.kwargs}
+        kwargs = {'request': request, 'excute': self._excute, 'error': error, 'response': response,
+                  'session': self.session, 'kwargs': self.kwargs, 'num': num}
         for i in inspect.getfullargspec(hook).args:
-            if i in ['self', 'cls']:
+            if i in ['self', 'cls'] or i not in kwargs:
                 continue
             kwarg[i] = kwargs[i]
-        hook(**kwarg)
-    
+        try:
+            logger.debug(f'执行hook：{hook}')
+            return hook(**kwarg)
+        except Exception as e:
+            if num <= 3:
+                for hook in self.except_hook_list:
+                    if self.hook_switch[hook.__name__] is True:
+                        return self.excute_kwargs(hook, request, error=e, num=num+1)
+                if len(self.except_hook_list) == 0:
+                    raise TypeError(f'未定义except_hook异常，异常内容:{traceback.format_exc()}')
+            else:
+                raise e
+
     def copy_request(self, request):
         new_request = {}
         if 'files' not in request:
@@ -145,7 +214,7 @@ class requestBase(api.Api):
                     new_request[key] = copy.deepcopy(request[key])
         return new_request
 
-    def _excute(self, request):
+    def _excute(self, request) -> requests.Response:
         current_time = time.time()
         new_request = self.copy_request(request)
         new_request['url'] = self.get_url(new_request.get('path'))
@@ -160,12 +229,23 @@ class requestBase(api.Api):
         for i in ['id', 'title']:
             if i in new_request:
                 del new_request[i]
-        result = self.session.request(**new_request)
-        response = result.text.replace('\n', '').replace('\r', '').replace('  ', '')
-        self.logger.debug('请求参数：method：{}，url：{}, \nparams:{}, headers:{}, \ndata:{}, json:{}, \n返回：{},时间:{}, '
-                          '状态码:{}'.format(new_request.get('method'), new_request.get('url'), new_request.get('params'),
-                                            new_request.get('headers'), new_request.get('data'), new_request.get('json'),
-                                            response, time.time() - current_time, result.status_code))
+        params = json.dumps(new_request.get('params'), indent=4, ensure_ascii=False, sort_keys=True)
+        data = json.dumps(new_request.get('data'), indent=4, ensure_ascii=False, sort_keys=True)
+        headers = json.dumps(new_request.get('headers'), indent=4, ensure_ascii=False, sort_keys=True)
+        _json = json.dumps(new_request.get('json'), indent=4, ensure_ascii=False, sort_keys=True)
+        msg = '请求参数：method：{}，url：{}, \nparams:{}, \nheaders:{}, \ndata:{}, \njson:{}'.format(new_request.get('method'),
+                                                                    new_request.get('url'), params, headers, data, _json)
+        try:
+            result = self.session.request(**new_request)
+        except requests.exceptions.ConnectionError as e:
+            self.logger.debug('{}, 入参:{}'.format(msg, new_request))
+            raise e
+        try:
+            response = json.dumps(result.json(), indent=4, ensure_ascii=False, sort_keys=True)
+        except:
+            response = result.text.replace('\n', '').replace('\r', '')
+        self.logger.debug('{}, \n返回：{},时间:{}, 状态码:{}'.format(msg, response, time.time() - current_time,
+                                                             result.status_code))
         if self.catch_response is True:
             return result
         return result
