@@ -10,6 +10,7 @@ import datetime
 import time
 from Crypto.Cipher import AES
 from ..common import logger
+# from loguru import logger
 
 
 class AESCipher(object):
@@ -72,16 +73,19 @@ class LarkMsg(object):
         elements.append({'tag': 'hr'})
         elements.append({"tag": "div", "fields": fields3, "text": {"tag": "lark_md", "content": "**执行详情**"}})
         for element in other_elements:
-            elements.append({'tag': 'hr'})
             fields_tmp = self.get_fileds(element['fields'])
-            elements.append({"tag": "div", "fields": fields_tmp,
-                             "text": {"tag": "lark_md", "content": "**{}**".format(element['title'])}})
+            if len(elements) > 0 and len(fields_tmp) > 0:
+                elements.append({'tag': 'hr'})
+                elements.append({"tag": "div", "fields": fields_tmp,
+                                 "text": {"tag": "lark_md", "content": "**{}**".format(element['title'])}})
 
         button = {"actions": [], "tag": "action"}
         for i in button_config:
-            button['actions'].append({"tag": "button", "text": {"content": i[0], "tag": "lark_md"},
-                                      "url": i[1], "type": "default", "value": {}})
-        elements.append(button)
+            if len(i) > 0:
+                button['actions'].append({"tag": "button", "text": {"content": i[0], "tag": "lark_md"},
+                                          "url": i[1], "type": "default", "value": {}})
+        if len(button_config) > 0:
+            elements.append(button)
         if is_at_all is True:
             elements.append({"tag": "markdown", "content": "<at id=all></at>"})
         elif len(at_list) > 0:
@@ -127,6 +131,18 @@ class LarkMsg(object):
             }
         }
 
+    def get_image(self, title, *img_keys):
+        data = {}
+        data["msg_type"] = "interactive"
+        config = {"wide_screen_mode": True, "enable_forward": True}
+        header = {"title": {"tag": "plain_text", "content": title}, "template": "orange"}
+        elements = []
+        for img_key in img_keys:
+            elements.append({"alt": {"content": "", "tag": "plain_text"}, "img_key": img_key, "tag": "img",
+                             "mode": "fit_horizontal", "compact_width": True})
+        data['card'] = {'config': config, 'elements': elements, 'header': header}
+        return data
+
     def get_fileds(self, _list):
         fields = []
         i = 0
@@ -157,6 +173,8 @@ class LarkSheet(object):
         """
         requests = [{'addSheet': {"properties": {"title": title, "index": index}}} for index, title in enumerate(titles)]
         response = self.edit_sheet("sheets_batch_update", {'requests': requests}).json()
+        if 'data' not in response:
+            raise TypeError(response['msg'])
         return [i["addSheet"]['properties']['sheetId'] for i in response['data']['replies']]
 
     def append_row(self, sheet_id, *values, style={}):
@@ -249,14 +267,46 @@ class LarkApi(object):
         data = self.msg.get_feishu_report_data(*args, **kwargs)
         return self.send_message(data)
 
-    def send_by_app(self, method, path, data=None, _json=None, params=None, is_app=True):
-        headers = {'Content-Type': 'application/json'}
+    def send_image(self, title, *path):
+        """
+        发送报告
+        :param title: 标题
+        :param img_key: 图片地址
+        :return:
+        """
+        img_key_list = [self.get_image_key(i) for i in path]
+        data = self.msg.get_image(title, *img_key_list)
+        logger.info(data)
+        return self.send_message(data)
+
+    def send_by_app(self, method, path, data=None, _json=None, params=None, is_app=True, **kwargs):
+        headers = {}
+        if 'files' not in  kwargs:
+            headers['Content-Type'] = 'application/json'
         if is_app is True:
             headers['Authorization'] = "Bearer " + self.get_tenant_access_token()
         url = self.url + path
-        response = requests.request(method, url, params=params, headers=headers, data=json.dumps(data), json=_json,
-                                verify=False)
-        logger.debug(f"url:{url}, \ndata:{data}, \nheaders:{headers}, params:{params}, \nresponse:{response.text}\n")
+        request = {}
+        request['headers'] = headers
+        request['url'] = url
+        request['method'] = method
+        request['verify'] = False
+        if data is not None:
+            if 'files' not in kwargs:
+                request['data'] = json.dumps(data)
+            else:
+                request['data'] = data
+        if _json is not None:
+            request['json'] = _json
+        if params is not None:
+            request['params'] = params
+        request.update(kwargs)
+        try:
+            response = requests.request(**request)
+            logger.debug(f"request:{request}, \nresponse:{response.text}\n")
+        except Exception as e:
+            logger.debug(f"request:{request}")
+            raise e
         return response
 
     def send(self, data):
@@ -292,6 +342,10 @@ class LarkApi(object):
         path = '/open-apis/sheets/v2/spreadsheets/{}/{}'.format(spreadsheetToken, name)
         return self.send_by_app(method, path, data, json)
 
+    def get_image_key(self, path):
+        return self.send_by_app("POST", '/open-apis/image/v4/put/', data={'image_type': 'message'},
+                                files={"image": open(path, 'rb')}, stream=True).json()['data']['image_key']
+
     def get_feishu_decrypt(self, encrypt):
         """
         解析飞书密文
@@ -321,6 +375,7 @@ class LarkApi(object):
 # sheet = LarkSheet("cli_a16d3c7ceb78d00a", "P4ntQsQ3L2c8MXPT1cRXtgVGoEk0iBLf", )
 #
 # robot = LarkApi("cli_a16d3c7ceb78d00a", "P4ntQsQ3L2c8MXPT1cRXtgVGoEk0iBLf")
+# print(robot.get_image_key(r'C:\Users\User\AppData\Local\Temp\tmpxwf0rajo\home.png'))
 # sheet = robot.get_sheet_object("shtusX0gWFaNl4YZOatQQyN2VSe")
 # for sheet_id in sheet.append_sheet("测试4"):
 #     logger.debug(sheet.append_row(sheet_id, ["标题", "时间", "test1"], ["123", "4", "56"]))
