@@ -17,11 +17,16 @@ class Conversion(object):
         self.start = r'\$(\d+)'
         self.end = r'\$'
         self.variable_regex = r"([a-zA-z].*)"
-        self.function_regex = r"\{(\w+)\((.*?)\)\}"
+        self.function_regex = r"\{(.*?)\((.*?)\)\}"
         self.json_regex = r"JSON((\[.*?\]){1,20})"
 
-    def re_dict(self):
-        re_order_list = self.get_re_dict(self.json)
+    def re_dict(self, re_black_list=[]):
+        new_json = copy.deepcopy(self.json)
+        if len(re_black_list) > 0:
+            for k in list(new_json.keys()):
+                if k in re_black_list:
+                    new_json.pop(k)
+        re_order_list = self.get_re_dict(new_json)
         re_order_list.sort()
         re_order_list = [i[1] for i in re_order_list]
         func_com = re.compile(self.start + self.function_regex)
@@ -32,11 +37,11 @@ class Conversion(object):
             for j in i:
                 key = key[j]
             new_value = copy.deepcopy(key)
-            if len(func_com.findall(key)) > 0:
+            if len(func_com.findall(new_value)) > 0:
                 new_value = self.re_sub_func(self.start + self.function_regex, new_value, self.replace_function)
-            if len(json_com.findall(key)) > 0:
+            if isinstance(new_value, str) and len(json_com.findall(new_value)) > 0:
                 new_value = self.re_sub_func(self.start + self.json_regex, new_value, self.replace_json)
-            if len(var_com.findall(key)) > 0:
+            if isinstance(new_value, str) and len(var_com.findall(new_value)) > 0:
                 new_value = self.re_sub_func(self.start + self.variable_regex, new_value, self.replace_variable)
             _json = self.json
             for jj in i[:-1]:
@@ -55,14 +60,16 @@ class Conversion(object):
                 pos = k.span()
                 try:
                     if (k.start() == 0 and len(value) == k.end()):
-                        new_value = json.loads(func(k))
+                        new_value = func(k)
                     else:
                         _str = list(new_value)
-                        _str[pos[0]:pos[1]] = new_v = str(json.loads(func(k)))
+                        _str[pos[0]:pos[1]] = new_v = str(func(k))
                         new_value = ''.join(_str)
                 except Exception as e:
                     logger.error('{} 替换异常:{}'.format(value, traceback.format_exc()))
                     raise e
+            if value == new_value:
+                raise TypeError(f'替换失败{value},未找到对应的方法或变量')
             logger.debug('替换特殊字符串：{}，替换结果:{}'.format(value, new_value))
         return new_value
 
@@ -117,12 +124,12 @@ class Conversion(object):
                 try:
                     from inspect import isgeneratorfunction
                     if isgeneratorfunction(obj) is False:
-                        return json.dumps(obj())
+                        return obj()
                     else:
-                        return json.dumps(next(obj()))
+                        return next(obj())
                 except:
                     logger.error('replace value:{},error:{}'.format(value, traceback.format_exc()))
-            return json.dumps(value.group())
+            return value.group()
         for arg in _find_list[1].split(','):
             if arg.find('=') != -1:
                 _tmp = arg.split('=')
@@ -157,9 +164,8 @@ class Conversion(object):
         obj = self.this.get_func(func)
         if obj is not False:
             result = obj(*arg, **kwargs)
-            result = json.dumps(result)
         else:
-            result = json.dumps(value.group(0))
+            result = value.group(0)
         return result
 
     def replace_json(self, value):
@@ -170,16 +176,16 @@ class Conversion(object):
         _find_list = _find_list[0]
         tmp = copy.deepcopy(self.json)
         for i in re.compile(r'\[(.*?)\]').findall(_find_list):
-            if type(tmp) == list:
+            if type(tmp) in [list, tuple, set]:
                 tmp = tmp[int(i)]
-            elif type(tmp) == str:
+            elif isinstance(tmp, str):
                 if i.find(':') != -1:
                     tmp = tmp[int(i.split(':')[0]):int(i.split(':')[1])]
                 else:
                     tmp = tmp[int(i)]
             else:
                 tmp = tmp[i]
-        return json.dumps(tmp)
+        return tmp
 
     def replace_variable(self, value):
         if len(value.groups()) > 1:
@@ -190,7 +196,7 @@ class Conversion(object):
         if variable.startswith('JSON') is True:
             tmp = copy.deepcopy(self.json)
             for i in re.compile(r'\[(.*?)\]').findall(variable):
-                if type(tmp) == list:
+                if type(tmp) in [list, tuple, set]:
                     tmp = tmp[int(i)]
                 elif type(tmp) == str:
                     if i.find(':') != -1:
@@ -199,11 +205,11 @@ class Conversion(object):
                         tmp = tmp[int(i)]
                 else:
                     tmp = tmp[i]
-            return json.dumps(tmp)
+            return tmp
         obj = self.this.get_func(variable)
         if obj is not False:
-            return json.dumps(obj)
-        return json.dumps(variable)
+            return obj
+        return variable
 
 
 #
@@ -249,7 +255,6 @@ class Test(object):
         self.current_subtle_unix_str = 123
         con = Conversion(self, ins)
         con.re_dict()
-        print(con.json)
 
     def get_cash_sweep(self, v, k):
         return 123
