@@ -1,5 +1,6 @@
 import copy
 import datetime
+import functools
 import json
 import os
 import sys
@@ -26,6 +27,7 @@ from .tool import yapi
 from autoTestScheme.configuration import robot
 from .common.logger import logger
 from . import conf
+import copy
 
 json_name = 'autoTestScheme'
 
@@ -38,20 +40,22 @@ def run_case(data: list, _uuid: str, env: str):
     _data['__env'] = env
     num = 1
     for i in data:
-        __file = i.get('__file')
-        __func = i.get('__func')
+        line = copy.deepcopy(i)
+        __file = line.get('__file')
+        __func = line.get('__func')
         if __file not in list(_data.keys()):
             _data[__file] = {}
         if __func not in list(_data[__file].keys()):
             _data[__file][__func] = []
-        i['__order'] = num
-        _data[__file][__func].append(i)
+        line['__order'] = num
+        _data[__file][__func].append(line)
         num += 1
     folder = os.path.join(constant.RUN_TMP_DIR, _uuid)
     if not os.path.exists(folder):
         os.makedirs(folder)
     _path = os.path.join(folder, 'case.json')
-    logger.info('执行数据：{}, 数据地址:{}'.format(_data, _path))
+    if conf.settings.get('is_hook_logger', True) is False:
+        logger.info('执行数据：{}, 数据地址:{}'.format(_data, _path))
     with open(_path, 'w+')as f:
         f.write(json.dumps(_data, sort_keys=True, indent=4, separators=(',', ':')))
 
@@ -59,7 +63,8 @@ def run_case(data: list, _uuid: str, env: str):
     command = ['-s', '--capture=no', '-vvv', '--alluredir', _path, '--clean-alluredir',
                '--cache-clear', '-W', 'ignore:Module already imported:pytest.PytestWarning', '--{}'.format(json_name),
                _uuid]
-    logger.info('执行命令:{}'.format(command))
+    if conf.settings.get('is_hook_logger', True) is False:
+        logger.info('执行命令:{}'.format(command))
     pytest.main(command)
 
 
@@ -75,21 +80,14 @@ class Run(object):
         self.all_data = []
         self.all_data_by_channel_id = {}
         self.uuid_list = []
+        self.log_path = None
         self.data_folder = None
         self.api_data = []
+        self.run_data = []
         self.result_folder = None
 
-    def register_tags(self, tags: dict):
-        """
-        注册标签列表
-        :param tags:
-        :return:
-        """
-        for key, value in tags.items():
-            self.register_tag(key, value)
-
     def set_logger(self, level="DEBUG", log_file_level="DEBUG", is_allure=True, logger_folder_name='logs',
-                   logger_start_str='run'):
+                   logger_start_str='run', is_time=True):
         """
         修改终端日志登记
         :param _str:
@@ -100,8 +98,11 @@ class Run(object):
         log_folder = os.path.join(constant.BASE_DIR, logger_folder_name)
         if not os.path.exists(log_folder):
             os.makedirs(log_folder)
-        log_path = os.path.join(log_folder, logger_start_str + '_{time}.log')
-        logger.add(log_path, level=log_file_level, rotation="200 MB")
+        if is_time is True:
+            self.log_path = os.path.join(log_folder, logger_start_str + '_{time}.log')
+        else:
+            self.log_path = os.path.join(log_folder, logger_start_str + '.log')
+        logger.add(self.log_path, level=log_file_level, rotation="200 MB")
         if is_allure is True:
             logger.add(PropogateHandler(), level=level, format="{time:YYYY-MM-DD at HH:mm:ss} | {message}")
 
@@ -148,44 +149,19 @@ class Run(object):
 
     def run(self):
         self.init_dir()
-        tag_name = ';'.join([conf.settings.run.tag_env_list[i] for i in conf.settings.run.test_tags])
-        logger.info('开始使用 {} 个进程 {} {}(标签:{})的 {} 用例'.format(conf.settings.run.process_num,
-                                                       '运行' if conf.settings.run.is_debug is False else '调试'
-                                                        , conf.settings.get_env_name(), tag_name,
-                                                       conf.settings.run.test_case))
-        logger.info('开始收集测试用例。。。。。')
+        logger.info('开始 {} {}(环境:{})的 {} 用例'.format('运行' if conf.settings.run.is_debug is False else '调试'
+                                                         , conf.settings.run.env, conf.settings.run.env_name,
+                                                         conf.settings.run.test_case))
+        if conf.settings.get('is_hook_logger', True) is False:
+            logger.info('开始收集测试用例。。。。。')
         group, surplus_data = self.collect_use_cases()
-
-        group_length = len(group)
-        surplus_length = len(surplus_data)
-
-        logger.info('分组用例：{}'.format(group))
-        logger.info('未分组用例：{}'.format(surplus_data))
-
-        # 执行分组用例
-        if len(group) == 0:
-            logger.info('分组用例为空')
-            del group
-        elif len(group) == 1:
-            self.run_case(group[0])
-        else:
-            self.start_process(self.list_transformation_queue(group))
-
-        # 执行未分组用例
-        if len(surplus_data) == 0:
-            logger.info('未分组用例为空')
-        elif (len(surplus_data) == 1 and len(
-                [j for i in surplus_data for j in i]) == 1) or conf.settings.run.process_num == 1:
-            for i in surplus_data:
-                self.run_case(i)
-        else:
-            for surplus in surplus_data:
-                self.start_process(self.list_transformation_queue(surplus))
-        if group_length == 0 and surplus_length == 0:
-            logger.info("结束测试。。。。")
-            return
-        self.collect_reports()
-        self.update_environment(constant.TMP_REPORT_DIR)
+        self.run_data = [j for i in group for j in i] + [j for i in surplus_data for j in i]
+        if conf.settings.get('is_hook_logger', True) is False:
+            logger.info(self.run_data)
+        if len(self.run_data) > 0:
+            self.run_case(self.run_data)
+            self.collect_reports()
+            self.update_environment(constant.TMP_REPORT_DIR)
         logger.error('运行完成....')
 
     def run_case(self, data):
@@ -193,50 +169,15 @@ class Run(object):
         self.uuid_list.append(_uuid)
         run_case(data, _uuid, conf.settings.current_env)
 
-    def start_process(self, group_queue):
-        total = group_queue.qsize()
-        pro_list = []
-        import threading
-        while len(pro_list) != 0 or group_queue.qsize() != 0:
-            for pro in pro_list:
-                if pro.is_alive() is False:
-                # if pro.isAlive() is False:
-                    name = pro.name
-                    pro_list.remove(pro)
-            if len(pro_list) < conf.settings.run.process_num and group_queue.qsize() > 0:
-                try:
-                    data = group_queue.get_nowait()
-                    _uuid = str(uuid.uuid4())
-                    self.uuid_list.append(_uuid)
-                    logger.info("执行进度:{}/{}".format(group_queue.qsize(), total))
-                    p = threading.Thread(target=run_case, args=(data, _uuid, conf.settings.current_env))
-                    p.setName(_uuid)
-                    p.start()
-
-                    # p = Process(target=run_case, args=(data, _uuid, conf.settings.current_env))
-                    # p.start()
-                    # p.name = _uuid
-                    pro_list.append(p)
-                except Exception:
-                    logger.error(traceback.format_exc())
-            time.sleep(2)
-
-    def list_transformation_queue(self, _list) -> queue.Queue:
-        _result = queue.Queue()
-        for i in list(range(len(_list)))[::-1]:
-            _result.put(_list[len(_list) - 1 - i])
-        del _list
-        return _result
-
     def collect_use_cases(self):
         group_data = []
         self.init_all_data()
         # 区分下渠道
         _all_data = {}
         for i in self.all_data:
-            if i.get('tag') not in list(_all_data.keys()):
-                _all_data[i.get('tag')] = []
-            _all_data[i.get('tag')].append(i)
+            if i.get('env') not in list(_all_data.keys()):
+                _all_data[i.get('env')] = []
+            _all_data[i.get('env')].append(i)
 
         for channel_data in list(_all_data.values()):
             channel_group_data = self.get_group_by_channel(copy.deepcopy(channel_data))
@@ -315,7 +256,7 @@ class Run(object):
                         filter = "@pytest.mark.skip(reason='got empty parameter set ['data']"
                         labels = _tmp.get_key('labels')
                         for label in labels:
-                            if label.get('name') == 'tag':
+                            if label.get('name') == 'env':
                                 if label.get('value').startswith(filter):
                                     if_copy = False
                                     break
@@ -338,7 +279,7 @@ class Run(object):
         获取所有用例数据
         :return:
         """
-        logger.info(self.data_folder)
+        # logger.info(self.data_folder)
         for root, dirs, files in os.walk(self.data_folder):
             for name in files:
                 _path = os.path.join(root, name)
@@ -353,26 +294,65 @@ class Run(object):
                         case = copy.deepcopy(__init)
                         case = common.NewDict(case)
                         case.merge_dict(i)
-                        for tag in list(set(conf.settings.run.test_tags).intersection(set(case.getdefault('tags', [])))):
-                            case['tag'] = tag
-                            for subTag in case.getdefault('subTags', ['default']):
-                                if case.get('id') is None:
-                                    case['id'] = str(uuid.uuid4())
-                                case['subTag'] = subTag
-                                for sonTag in case.getdefault('sonTags', ['default']):
-                                    case['sonTag'] = sonTag
-                                    if tag not in list(self.all_data_by_channel_id.keys()):
-                                        self.all_data_by_channel_id[tag] = {}
-                                    if case['id'] not in self.all_data_by_channel_id[tag]:
-                                        self.all_data_by_channel_id[tag][case['id']] = []
-                                    self.all_data_by_channel_id[tag][case['id']].append(copy.deepcopy(case))
-                                    self.all_data.append(copy.deepcopy(case))
+                        for env in list(set([conf.settings.run.env]).intersection(set(case.getdefault('env', [])))):
+                            case['env'] = env
+                            if case.get('id') is None:
+                                case['id'] = str(uuid.uuid4())
+                            if env not in list(self.all_data_by_channel_id.keys()):
+                                self.all_data_by_channel_id[env] = {}
+                            if case['id'] not in self.all_data_by_channel_id[env]:
+                                self.all_data_by_channel_id[env][case['id']] = []
+
+                            tag_case_list = self.calculation_tag_case(case)
+                            print(tag_case_list)
+
+                            self.all_data_by_channel_id[env][case['id']] += tag_case_list
+                            self.all_data += tag_case_list
+                            # self.all_data_by_channel_id[env][case['id']].append(copy.deepcopy(case))
+                            # self.all_data.append(copy.deepcopy(case))
         self.replace_base_data()
 
+    def calculation_tag_case(self, case):
+        def cmp(a1, a2):
+            return int(a1.replace('tag', '')) < int(a2.replace('tag', ''))
+        tag_list = []
+        for i in case:
+            if i.startswith('tag'):
+                tag_list.append(i)
+        if len(tag_list) == 0:
+            return [case]
+        tag_list.sort(key=functools.cmp_to_key(cmp))
+        tags = [case[i] for i in tag_list]
+        total = functools.reduce(lambda x, y: x * y, map(len, tags))
+        retList = []
+        for i in range(0, total):
+            step = total
+            tempItem = []
+            for l in tags:
+                step /= len(l)
+                tempItem.append(l[int(i / step % len(l))])
+            tmp = copy.deepcopy(case)
+            for i, line in enumerate(tempItem):
+                tmp[f'tag{i}'] = line
+            retList.append(tmp)
+        return retList
+
+    def myfunc(*lists):
+        total = functools.reduce(lambda x, y: x * y, map(len, lists))
+        retList = []
+        for i in range(0, total):
+            step = total
+            tempItem = []
+            for l in lists:
+                step /= len(l)
+                tempItem.append(l[int(i / step % len(l))])
+            retList.append(tempItem)
+        return retList
+
     def get_dependent_data(self, data):
         result = []
         dependent = data.get('dependent')
-        channel = data.get('tag')
+        channel = data.get('env')
         if channel in list(self.all_data_by_channel_id.keys()) and \
                 dependent in list(self.all_data_by_channel_id[channel].keys()):
             _dep_list = self.all_data_by_channel_id[channel].get(dependent)
@@ -389,6 +369,7 @@ class Run(object):
         :return:
         """
         _num = 0
+        # print(self.all_data)
         for i in range(len(self.all_data)):
             data = self.all_data[i]
             _base = data.get('__base')
@@ -401,24 +382,24 @@ class Run(object):
                 for k, j in v.items():
                     if k not in list(_base_data.keys()):
                         _base_data[k] = j
-            channel = data.get('tag')
+            channel = data.get('env')
             _id = data.get('id')
             if channel in list(self.all_data_by_channel_id.keys()) and _base in list(
                     self.all_data_by_channel_id[channel].keys()):
                 data = self.merge_dict(copy.deepcopy(self.all_data_by_channel_id[channel][_base]), copy.deepcopy(data))
                 del data['__base']
                 self.all_data[i] = data
-                self.all_data_by_channel_id[data['tag']][data.get('id')] = data
+                self.all_data_by_channel_id[data['env']][data.get('id')] = data
             elif _base in list(_base_data.keys()):
                 data = self.merge_dict(copy.deepcopy(_base_data[_base]), copy.deepcopy(data))
                 del data['__base']
                 self.all_data[i] = data
-                self.all_data_by_channel_id[data['tag']][data.get('id')] = data
+                self.all_data_by_channel_id[data['env']][data.get('id')] = data
             elif _base is not None:
-                logger.info("数据对应base不存在:{}".format(_base))
+                logger.error("数据对应base不存在:{}".format(_base))
                 self.all_data[i] = {}
                 _num += 1
-                self.all_data_by_channel_id[data['tag']][data.get('id')] = data
+                self.all_data_by_channel_id[data['env']][data.get('id')] = data
 
         for i in range(_num):
             self.all_data.remove({})
@@ -434,7 +415,7 @@ class Run(object):
 
     def check_and_create_by_dir(self, dir):
         if not os.path.exists(dir):
-            logger.debug('创建目录:{}'.format(dir))
+            # logger.debug('创建目录:{}'.format(dir))
             os.makedirs(dir)
 
     def init_dir(self):
@@ -483,13 +464,16 @@ class Run(object):
         _time = _config.get('time')
         return statistic, _time
 
-    def get_report(self,*result_folder):
+    def get_report(self, *result_folder):
         """
         生成allure报告
         :param result_folder:报告地址文件名
         :return:
         """
         self.result_folder = os.path.join(constant.BASE_DIR, *result_folder)
+        if len(self.run_data) == 0:
+            logger.error("请执行用例之后生成allure报告")
+            return
         command = '{} generate {} -c -o {}'.format(constant.ALLURE_PATH, constant.TMP_REPORT_DIR, self.result_folder)
         logger.info(command)
         self.update_environment(constant.TMP_REPORT_DIR)
